```cpp
ll fpow(ll a,ll n,ll mod = 1e9 + 7)
{
    ll res = 1;
    while(n)
    {
        if(n & 1)
        {
            res = res % mod * a % mod;
        }
        a = a % mod * a % mod;
        n >>= 1;
    }
    return res % mod; 
}

ll fac[maxn];
ll inv[maxn];

void getfacinv(int n, ll mod = 1e9 + 7)
{//初始化fac和inv数组，获取1!-n!和相应的逆元
    fac[0] = inv[0] = 1;
    for (int i = 1; i <= n; i++) fac[i] = fac[i - 1] * i % mod;
    //需要快速比较大小lg[i] = lg[i - 1] + log(i);
    inv[n] = fpow(fac[n], mod - 2, mod);
    for (int i = n - 1; i >= 1; i--) inv[i] = inv[i + 1] * (i + 1LL) % mod;
}

ll com(ll a, ll b, ll mod = 1e9 + 7)
{
    //计算组合数
    if (a < b) return 0;
    return fac[a] * inv[b] % mod * inv[a-b] % mod;
}

struct comnums
{//C(a,b) a>=b
    int a, b;
    bool operator <(comnums s)const
    {//组合数大小比较，比较对数
        return lg[a] - lg[b] - lg[a - b] < lg[s.a] - lg[s.b] - lg[s.a - s.b];
    }
};
```

## 范德蒙德卷积

$\displaystyle \sum_{i=0}^{k} C_{x}^{i}\times C_{y}^{k-a-i}=C_{x+y}^{k}$

即从 $x$ 中选择 $i$ 个，在 $y$ 中选择 $k-i$ 个相当于在 $x+y$ 中选择 $k$ 个的方案数

#### 例

求 $\displaystyle \sum_{i=0}^{x}C_{x}^{i}C_{y}^{z+i}mod\quad 998244353$

$(0\leq x,y,z \leq 1000000,\quad z\leq y)$

原式可以变为

$\displaystyle \sum_{i=0}^{x}C_{x}^{x-i}C_{y}^{z+i}$

此式根据范德蒙德卷积等于 $C_{x+y}^{x+z}$

### 拓展

$\displaystyle \sum_{i=1}^{n}C_{n}^{i}C_{n}^{i+1}=\sum_{i=0}^{n-1}C_{n}^{i}C_{n}^{i-1}=C_{2n}^{n-1}$

$\displaystyle \sum_{i=1}^{n}C_{n}^{i}C{n}^{i-1}=\sum_{i=0}^{n-1}C_{n}^{i}C_{n}^{i+1}=\sum_{i=0}^{n-1}C_{n}^{i}C_{n}^{n-(n-1-i)}=C_{2n}^{n}$

$\displaystyle \sum_{i=0}^{n}{C_{n}^{i}}^2=C_{2n}^{n}$

$\displaystyle \sum_{i=0}^{m}C_{n}^{i}C_{m}^{i}=C_{n+m}^{m}$