定理：$F(n)$ 和 $f(n)$ 是定义在非负整数集合上的两个函数，并且满足

$\displaystyle F(n)=\sum_{d|n}f(d)$

那么存在一个结论：

$\displaystyle f(n)=\sum_{d|n}\mu(d)F(\left \lfloor \frac{n}{d}  \right \rfloor )$

另一种形式：当 $F(n)$ 和 $f(n)$ 满足

$\displaystyle F(n)=\sum_{n|d} f(d)$

可以推出

$\displaystyle f(n)=\sum_{n|d} \mu(\frac{d}{n})F(d)$



## P2257 YY的GCD

给定 $n,m$ 求 $\displaystyle \sum_{i=1}^{n}\sum_{i=1}^{m}[\gcd(i,j)\in{prime}]$

与 $\gcd$ 有关的莫比乌斯反演，一般设 $f(d)$ 为 $\gcd(i,j)=d$ 的个数，即 

$\displaystyle f(d)=\sum_{i=1}^{n}\sum_{j=1}^{m}[gcd(i,j)=d]$

然后设 $F(n)$ 为 $\gcd$ 是 $n$ 及 $n$ 的倍数的数量，即 

$\displaystyle F(x)=\sum_{i=1}^{n}\sum_{j=1}^{m}[\gcd(i,j)|x]=\left \lfloor  \frac{n}{x}\right\rfloor\left\lfloor\frac{m}{x}\right \rfloor $

由此可得

$\displaystyle F(x)=\sum_{x|d}f(d)$

由莫比乌斯反演，该式等价于

$\displaystyle f(n)=\sum_{n|d}\mu(\left\lfloor\frac{d}{n}\right\rfloor)F(d)$

由此开始首先根据题意

$\displaystyle ans=\sum_{p\in{prime}}\sum_{i=1}^{n}\sum_{j=1}^{m}[\gcd(i,j)=p]$

代入 $f(p)$

$\displaystyle ans=\sum_{p\in{prime}} f(p)$

代入莫比乌斯反演得到的公式

$\displaystyle ans=\sum_{p\in{prime}}\sum_{p|d}\mu(\left\lfloor\frac{d}{p}\right\rfloor)F(d)$

将连加项替换为 $\displaystyle\left\lfloor\frac{d}{p}\right\rfloor$

$\displaystyle ans=\sum_{p\in{prime}}\sum_{d=1}^{\min(\left\lfloor\frac{n}{p}\right\rfloor,\left\lfloor\frac{m}{p}\right\rfloor)}\mu(d)F(dp)$

代入 $\displaystyle F(dp)=\left\lfloor\frac{n}{dp}\right\rfloor\left\lfloor\frac{m}{dp}\right\rfloor$

$\displaystyle ans=\sum_{p\in{prime}}\sum_{d=1}^{\min(\left\lfloor\frac{n}{p}\right\rfloor,\left\lfloor\frac{m}{p}\right\rfloor)}\mu(d)\left\lfloor\frac{n}{dp}\right\rfloor\left\lfloor\frac{m}{dp}\right\rfloor$

设 $T=dp$

$\displaystyle ans=\sum_{T=1}^{\min(n,m)}\sum_{t|T,t\in{prime}}\mu(\left\lfloor\frac{T}{t}\right\rfloor)\left\lfloor\frac{n}{T}\right\rfloor\left\lfloor\frac{m}{T}\right\rfloor$

将$\displaystyle \left\lfloor\frac{n}{T}\right\rfloor\left\lfloor\frac{m}{T}\right\rfloor$ 提出

$\displaystyle ans=\sum_{T=1}^{\min(n,m)}\left\lfloor\frac{n}{T}\right\rfloor\left\lfloor\frac{m}{T}\right\rfloor\sum_{t|T,t\in{prime}}\mu(\left\lfloor\frac{T}{t}\right\rfloor)$

前半部分的求和可以用数论分块。后半部分预处理对莫比乌斯函数作前缀和。可以在 $O(\sqrt{\min(n,m)})$ 的时间内求出每个样例的答案。

```cpp
#include<iostream>
#include<string>
#include<queue>
#include<stack>
#include<vector>
#include<map>
#include<cstdio>
#include<cstdlib>
#include<algorithm>
#include<set>
#include<list>
#include<iomanip>
#include<cstring>
#include<cmath>
#include<limits>
#include<cctype>
#include<unordered_map>
#include<complex>
#include<assert.h>
#pragma warning(disable:4996)
using namespace std;

#define debug(i) cerr<<"<debug> "<<#i<<" "<<i<<" <\\debug>"<<endl
#define mfor(i,a,b) for(int i=(a);i<=(b);i++)
#define mrep(i,a,b) for(int i=(a);i>=(b);i--)
#define _for(i,a,b) for(ll i=(a);i<=(b);i++)
#define _rep(i,a,b) for(ll i=(a);i>=(b);i--)
#define lll __int128
#define Re register
#define mem(a,b) memset(a,(b),sizeof(a))
#define Design ios::sync_with_stdio(0),cin.tie(0),cout.tie(0)
#define endl '\n'
#define toang(x) ((x)*180.0/pi)	//转角度
#define torad(x) ((x)*pi/180.0)	//转弧度
typedef long long ll;
typedef unsigned long long ull;
typedef pair<int, int> intpair;
typedef pair<ll, ll> llpair;
typedef long double ld;
typedef double dl;
const dl pi = acos(-1.0);
const dl eps = 1e-5;
const int INF = 0x3f3f3f3f;
const ll INFLL = 0x3f3f3f3f3f3f3f3f;

const int maxn = 1e7 + 10;

struct GP_mu
{
	bool np[maxn];
	vector<int>prime;
	int mu[maxn];
	int n;
	GP_mu(int n) :n(n) {}
	void getprime()
	{
		memset(np, 0, sizeof(np));
		np[0] = np[1] = true;
		mu[1] = 1;
		for (int i = 2; i <= n; i++)
		{
			if (!np[i])	prime.emplace_back(i), mu[i] = -1;
			for (auto it : prime)
			{
				if (i * it > n) break;
				np[i * it] = true;
				if (i % it == 0)
				{
					mu[i * it] = 0;
					break;
				}
				mu[i * it] = -mu[i];
			}
		}
	}
}gm(1e7);

ll g[10000010];
ll sum[10000010];

int main()
{
	gm.getprime();
	for (auto it : gm.prime)
	{
		for (ll i = 1; i * it <= 1e7; i++)
		{
			g[i * it] += gm.mu[i];
		}
	}
	for (int i = 1; i <= 1e7; i++) sum[i] = sum[i - 1] + g[i];
	int T;
	cin >> T;
	while (T--)
	{
		int n, m;
		scanf("%d%d", &n, &m);
		if (n > m) swap(n, m);
		ll ans = 0;
		for (ll l = 1, r; l <= n; l = r + 1)
		{
			r = min(n / (n / l), m / (m / l));
			ans += (n / l) * (m / l) * (sum[r] - sum[l - 1]);
		}
		printf("%lld\n", ans);
	}
    return 0;
}
```

## 2020吉林省赛H. Curious

给定 $n,m,k$ 对于 $k$ 个给定 $x$ 求 $\displaystyle \sum_{i=1}^{n}\sum_{j=1}^{n}[\gcd(a_i,a_j)=x]$

如同上面的套路，设 $f(d)$ 表示 $\gcd(a_i,a_j)=d$ 的数量

$\displaystyle f(d)=\sum_{i=1}^{n}\sum_{j=1}^{m}[gcd(a_i,a_j)=d]$

设 $F(n)$ 表示满足 $\gcd(a_i,a_j)|n$ 的数量

$\displaystyle F(n)=\sum_{d|n}f(d)$

通过 $\displaystyle \sum_{i=1}^{n}\sum_{j=1}^{n}gcd(a_i,a_j)|n$ 等价于 $\displaystyle\sum_{d|n}\sum_{i=1}^{n}\sum_{j=1}^{n}(d|a_i)\and (d|a_j)$ （$\and$ 表示合取即与）

可以得到 $\displaystyle F(n)=(\sum_{i=1}^{n}[x|a_i])^2$

由莫比乌斯反演

$\displaystyle f(x)=\sum_{d|x}\mu(\frac{d}{x})F(d)$

$F(d)$ 可以在 $n(1+\frac{1}{2}+\frac{1}{3}+....+\frac{1}{n})=O(nlog(n))$ 的调和级数复杂度下求出

$\mu(x)$ 可以使用线性筛 $O(n)$ 求出。

由此可以在 $O(nlog(n))$ 预处理出 $f(x)|x\in[1,m]$，即所需的答案。

```cpp
#include<iostream>
#include<string>
#include<queue>
#include<stack>
#include<vector>
#include<map>
#include<cstdio>
#include<cstdlib>
#include<algorithm>
#include<set>
#include<list>
#include<iomanip>
#include<cstring>
#include<cmath>
#include<limits>
#include<cctype>
#include<unordered_map>
#include<complex>
#include<assert.h>
#pragma warning(disable:4996)
using namespace std;

#define debug(i) cerr<<"<debug> "<<#i<<" "<<i<<" <\\debug>"<<endl
#define mfor(i,a,b) for(int i=(a);i<=(b);i++)
#define mrep(i,a,b) for(int i=(a);i>=(b);i--)
#define _for(i,a,b) for(ll i=(a);i<=(b);i++)
#define _rep(i,a,b) for(ll i=(a);i>=(b);i--)
#define lll __int128
#define Re register
#define mem(a,b) memset(a,(b),sizeof(a))
#define Design ios::sync_with_stdio(0),cin.tie(0),cout.tie(0)
#define endl '\n'
#define toang(x) ((x)*180.0/pi)	//转角度
#define torad(x) ((x)*pi/180.0)	//转弧度
typedef long long ll;
typedef unsigned long long ull;
typedef pair<int, int> intpair;
typedef pair<ll, ll> llpair;
typedef long double ld;
typedef double dl;
const dl pi = acos(-1.0);
const dl eps = 1e-5;
const int INF = 0x3f3f3f3f;
const ll INFLL = 0x3f3f3f3f3f3f3f3f;

const int maxn = 1e5 + 10;

struct GP_mu
{
	bool np[maxn];
	vector<int>prime;
	int mu[maxn];
	int n;
	GP_mu(int n) :n(n) {}
	void getprime()
	{
		memset(np, 0, sizeof(np));
		np[0] = np[1] = true;
		mu[1] = 1;
		for (int i = 2; i <= n; i++)
		{
			if (!np[i])	prime.emplace_back(i), mu[i] = -1;
			for (auto it : prime)
			{
				if (i * it > n) break;
				np[i * it] = true;
				if (i % it == 0)
				{
					mu[i * it] = 0;
					break;
				}
				mu[i * it] = -mu[i];
			}
		}
	}
}gm(1e5);

int a[200010];
ll tot[200010];
ll cnt[200010];
ll ans[200010];

int main()
{
	Design;
	gm.getprime();
	int T;
	cin >> T;
	while (T--)
	{
		mem(cnt, 0);
		mem(tot, 0);
		int n, m, k;
		cin >> n >> m >> k;
		ll t = 0;
		mfor(i, 1, n) cin >> a[i], cnt[a[i]]++;
		mfor(i, 1, m)
		{
			for (int j = i; j <= m; j += i) 
			{
				tot[i] += cnt[j];
			}
		}
		mfor(i, 1, m)
		{
			ans[i] = 0;
			for (int j = i; j <= m; j += i)
			{
				ans[i] += tot[j] * tot[j] * gm.mu[j / i];
			}
		}
		while (k--)
		{
			int x;
			cin >> x;
			cout << ans[x] << endl;
		}
	}
    return 0;
}
```

## AcWing 214. Devu和鲜花

给定$n,m,d$ ，求 $\displaystyle \sum_{i=1}^{n}\sum_{j=1}^{m}[\gcd(i,j)=d]$

多组样例

设 $f(n)$ 表示满足 $\gcd=d$ 的数字对数

$\displaystyle f(n)=\sum_{i=1}^{n}\sum_{j=1}^{m}[\gcd(i,j)=d]$

设 $F(n)$ 表示满足 $\gcd|d$ 的数字对数

$\displaystyle F(x)=\sum_{i=1}^{n}\sum_{j=1}^{m}[\gcd(i,j)|x]=\left\lfloor\frac{n}{x}\right\rfloor\left\lfloor\frac{m}{x}\right\rfloor$

有

$\displaystyle F(x)=\sum_{d|x}f(d)$

根据莫比乌斯反演有

$\displaystyle f(d)=\sum_{d|x}\mu(\frac{x}{d})F(x)=\sum_{d|x}\mu(\frac{x}{d})\left\lfloor\frac{n}{x}\right\rfloor\left\lfloor\frac{m}{x}\right\rfloor$

由此可以用数论分块和莫比乌斯函数的前缀和在 $O(\sqrt{\min(n,m)})$ 复杂度内求出结果

```cpp
#include<bits/stdc++.h>
using namespace std;

using ll=long long;
using ull=unsigned long long;

struct GP_mu {
	bool np[500010];
	vector<int>prime;
	int mu[500010];
	int n;
	GP_mu(int n) :n(n) {}
	void getprime() {
		memset(np,0,sizeof(np));
		np[0] = np[1] = true;
		mu[1]=1;
		for (int i = 2; i <= n; i++) {
			if (!np[i])	prime.emplace_back(i),mu[i]=-1;
			for (auto it : prime) {
				if (i * it > n) break;
				np[i * it] = true;
				if (i % it == 0) {
					mu[i*it]=0;
					break;
				}
				mu[i*it]=-mu[i];
			}
		}
	}
} gm(5e5);

int sum[500010];

int main() {
	gm.getprime();
	int T;
	scanf("%d",&T);
	for(int i=1;i<=5e5;i++) sum[i]=sum[i-1]+gm.mu[i];
	while(T--) {
		int n,m,d;
		scanf("%d%d%d",&n,&m,&d);
		int ans=0;
		if(n > m) swap(n, m);
		n/=d,m/=d;
		for(int l = 1, r; l <= n; l = r+1) {
			r = min(n/(n/l), m/(m/l));
			ans += ((n/l)*(m/l) * (sum[r] - sum[l-1]));
		}
		printf("%d\n", ans);
	}
	return 0;
}
```

